@@ -6,25 +6,28 @@ use log::{debug, info};
 use pathdiff::diff_paths;
 use structopt::StructOpt;
 
-use crate::api_types::ApiType;
+use crate::commands::ensure_tools_available;
 use crate::config::RawOpts;
+use crate::ir::*;
 use crate::others::*;
 use crate::utils::*;
 
-mod api_types;
 mod commands;
 mod config;
-mod generator_c;
-mod generator_dart;
-mod generator_rust;
+mod generator;
+mod ir;
+mod markers;
 mod others;
 mod parser;
+mod source_graph;
 mod transformer;
 mod utils;
 
 fn main() {
     env_logger::Builder::from_env(Env::default().default_filter_or("info")).init();
 
+    ensure_tools_available();
+
     let config = config::parse(RawOpts::from_args());
     info!("Picked config: {:?}", &config);
 
@@ -37,29 +40,28 @@ fn main() {
     let file_ast = syn::parse_file(&source_rust_content).unwrap();
 
     info!("Phase: Parse AST to IR");
-    let raw_api_file = parser::parse(&source_rust_content, file_ast);
-    debug!("parsed functions: {:?}", &raw_api_file);
+    let raw_ir_file = parser::parse(&source_rust_content, file_ast, &config.manifest_path);
+    debug!("parsed functions: {:?}", &raw_ir_file);
 
     info!("Phase: Transform IR");
-    let api_file = transformer::transform(raw_api_file);
-    debug!("transformed functions: {:?}", &api_file);
+    let ir_file = transformer::transform(raw_ir_file);
+    debug!("transformed functions: {:?}", &ir_file);
 
     info!("Phase: Generate Rust code");
-    let generated_rust = generator_rust::generate(
-        &api_file,
+    let generated_rust = generator::rust::generate(
+        &ir_file,
         &mod_from_rust_path(&config.rust_input_path, &config.rust_crate_dir),
     );
     fs::create_dir_all(&rust_output_dir).unwrap();
     fs::write(&config.rust_output_path, generated_rust.code).unwrap();
 
     info!("Phase: Generate Dart code");
-    let (generated_dart_file_prelude, generated_dart_decl_raw, generated_dart_impl_raw) =
-        generator_dart::generate(
-            &api_file,
-            &config.dart_api_class_name(),
-            &config.dart_api_impl_class_name(),
-            &config.dart_wire_class_name(),
-        );
+    let generated_dart = generator::dart::generate(
+        &ir_file,
+        &config.dart_api_class_name(),
+        &config.dart_api_impl_class_name(),
+        &config.dart_wire_class_name(),
+    );
 
     info!("Phase: Other things");
 
@@ -69,11 +71,11 @@ fn main() {
         others::try_add_mod_to_lib(&config.rust_crate_dir, &config.rust_output_path);
     }
 
-    let c_struct_names = api_file
+    let c_struct_names = ir_file
         .distinct_types(true, true)
         .iter()
         .filter_map(|ty| {
-            if let ApiType::StructRef(_) = ty {
+            if let IrType::StructRef(_) = ty {
                 Some(ty.rust_wire_type())
             } else {
                 None
@@ -97,7 +99,7 @@ fn main() {
                 temp_dart_wire_file.path().as_os_str().to_str().unwrap(),
                 &config.dart_wire_class_name(),
                 c_struct_names,
-                &config.llvm_path,
+                &config.llvm_path[..],
                 &config.llvm_compiler_opts,
             );
         },
@@ -108,7 +110,7 @@ fn main() {
         EXTRA_EXTERN_FUNC_NAMES.to_vec(),
     ]
     .concat();
-    let c_dummy_code = generator_c::generate_dummy(&effective_func_names);
+    let c_dummy_code = generator::c::generate_dummy(&effective_func_names);
     fs::create_dir_all(c_output_dir).unwrap();
     fs::write(
         &config.c_output_path,
@@ -125,8 +127,8 @@ fn main() {
 
     sanity_check(&generated_dart_wire.body, &config.dart_wire_class_name());
 
-    let generated_dart_decl_all = generated_dart_decl_raw;
-    let generated_dart_impl_all = &generated_dart_impl_raw + &generated_dart_wire;
+    let generated_dart_decl_all = generated_dart.decl_code;
+    let generated_dart_impl_all = &generated_dart.impl_code + &generated_dart_wire;
     if let Some(dart_decl_output_path) = &config.dart_decl_output_path {
         let impl_import_decl = DartBasicCode {
             import: format!(
@@ -141,18 +143,18 @@ fn main() {
         };
         fs::write(
             &dart_decl_output_path,
-            (&generated_dart_file_prelude + &generated_dart_decl_all).to_text(),
+            (&generated_dart.file_prelude + &generated_dart_decl_all).to_text(),
         )
         .unwrap();
         fs::write(
             &config.dart_output_path,
-            (&generated_dart_file_prelude + &impl_import_decl + &generated_dart_impl_all).to_text(),
+            (&generated_dart.file_prelude + &impl_import_decl + &generated_dart_impl_all).to_text(),
         )
         .unwrap();
     } else {
         fs::write(
             &config.dart_output_path,
-            (&generated_dart_file_prelude + &generated_dart_decl_all + &generated_dart_impl_all)
+            (&generated_dart.file_prelude + &generated_dart_decl_all + &generated_dart_impl_all)
                 .to_text(),
         )
         .unwrap();
